"""
just convert a markdown file to HTML and write it out to a file
"""
import os
import base64
from pathlib import Path
import argparse
import re
from markdown_it import MarkdownIt
from bs4 import BeautifulSoup


def markdown_to_html(
    MarkdownIt_obj,
    in_file_path,
    out_file_path="tmp.html",
    encode_local_images=False,
    css_file=None,
    live_md_rr=None,
):
    text = open(in_file_path, "r").read()
    tokens = MarkdownIt_obj.parse(text)
    html_text = MarkdownIt_obj.render(text)
    # pretty CSS
    soup = BeautifulSoup(html_text, "html5lib")  # adds <html>, <head>,  <body>
    soup.select_one("head").append(soup.new_tag("meta"))
    soup.select_one("meta").attrs["charset"] = "UTF-8"

    # syntax highlighting
    soup.select_one('head').append(soup.new_tag('link'))
    soup.select_one("link").attrs["rel"] = "stylesheet"
    soup.select_one("link").attrs["href"] = "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.9.0/styles/monokai.min.css"

    soup.select_one('head').append(soup.new_tag('script'))
    soup.select("script")[0].attrs['src'] ='https://cdnjs.cloudflare.com/ajax/libs/highlight.js/11.9.0/highlight.min.js'
    soup.select_one('head').append(soup.new_tag('script'))
    soup.select("script")[1].string = 'hljs.highlightAll();'

    soup.select_one("head").append(soup.new_tag("style"))
    if css_file:
        if css_file == "none":
            css = ""
        else:
            with open(css_file, "r") as f:
                css = f.read()
    else:
        css = """
body { background-color: #272822; color: #e6edf3; font-family: -apple-system,BlinkMacSystemFont,"Segoe UI","Noto Sans",Helvetica,Arial,sans-serif,"Apple Color Emoji","Segoe UI Emoji"; position: relative; max-width: 960px; margin: auto; line-height: 1.5; }
a[href] { color: #66d9ef; }
code { color: #e6edf3; background-color: #343941; font-family: monospace; padding: .2em .4em; border-radius: 6px; }
pre { padding: 1em; border-radius: 6px; background-color: #161b22; overflow-x:auto; }
pre > code { background-color: #161b22; padding: 0px 0px; }
table, th, td { border: 1px solid; border-collapse: collapse; padding-left: 4px; padding-right: 4px; }
img { max-width: 100%; }
"""
    soup.select_one("style").string = css
    if live_md_rr:
        script = (
            f"setTimeout(function(){{ document.location.reload(); }}, {live_md_rr});"
        )
        soup.select_one("head").append(soup.new_tag("script"))
        soup.select("script")[-1].string = script
    if encode_local_images:
        img_elems = soup.select("img")
        url_pattern = "^https?:\\/\\/(?:www\\.)?[-a-zA-Z0-9@:%._\\+~#=]{1,256}\\.[a-zA-Z0-9()]{1,6}\\b(?:[-a-zA-Z0-9()@:%_\\+.~#?&\\/=]*)$"
        for img in img_elems:
            if not re.match(url_pattern, img.attrs["src"]):
                img_path = os.path.join(os.path.dirname(in_file_path), img.attrs["src"])
                with open(img_path, "rb") as image_obj:
                    img_bytes = str(base64.b64encode(image_obj.read()), "utf-8")
                img.attrs["src"] = f"data:image/png;base64,{img_bytes}"
    Path(out_file_path).write_text(str(soup))


if __name__ == "__main__":
    MarkdownIt_obj = MarkdownIt("commonmark").enable("table").enable("strikethrough")
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--in_file_path",
        "-i",
        required=True,
        help="in-file-path; your existing markdown file",
    )
    parser.add_argument(
        "--out_file_path",
        "-o",
        default=None,
        help="out-file-path; your HTML file to be created",
    )
    parser.add_argument(
        "--encode_local_images",
        "-e",
        action="store_true",
        help="in HTML, embed base64-encoded data of local images linked to in your markdown; removes dependency on presence of the external local images",
    )
    parser.add_argument(
        "--css_file",
        default=None,
        help='css-file-path whose content will be written to the <style> element. Can be "none"; do not use any css',
    )
    args = parser.parse_args()

    if not args.out_file_path:
        args.out_file_path = str(Path(args.in_file_path).with_suffix('.html'))

    markdown_to_html(
        MarkdownIt_obj=MarkdownIt_obj,
        in_file_path=args.in_file_path,
        out_file_path=args.out_file_path,
        encode_local_images=args.encode_local_images,
        css_file=args.css_file,
    )
